import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import flatten from 'lodash/flatten';
import ColumnGroup from '../ColumnGroup';
/**
 * Get the columns ReactElement array.
 * - Handling the case where there is an array of <Column> in children.
 * - Filter empty items in children.
 */

function getTableColumns(children) {
  if (!Array.isArray(children)) {
    return children;
  }

  var flattenColumns = flatten(children).map(function (column) {
    if ((column === null || column === void 0 ? void 0 : column.type) === ColumnGroup) {
      var _column$props = column === null || column === void 0 ? void 0 : column.props,
          header = _column$props.header,
          childColumns = _column$props.children,
          align = _column$props.align,
          fixed = _column$props.fixed,
          verticalAlign = _column$props.verticalAlign,
          groupHeaderHeight = _column$props.groupHeaderHeight;

      return childColumns.map(function (childColumn, index) {
        // Overwrite the props set by ColumnGroup to Column.
        var groupCellProps = _extends({}, childColumn === null || childColumn === void 0 ? void 0 : childColumn.props, {
          groupHeaderHeight: groupHeaderHeight,
          align: align,
          fixed: fixed,
          verticalAlign: verticalAlign
        });
        /**
         * Set attributes for the first column in the group:
         * @field groupCount: The number of grouping sub-items.
         * @field groupHeader: Group header title.
         * @field resizable: Set to not resizable.
         */


        if (index === 0) {
          groupCellProps.groupCount = childColumns.length;
          groupCellProps.groupHeader = header;
          groupCellProps.resizable = false;
        }

        return /*#__PURE__*/React.cloneElement(childColumn, groupCellProps);
      });
    }

    return column;
  }); // Flatten the array in Columns into a one-dimensional array, and calculate lastColumn and firstColumn.

  return flatten(flattenColumns).filter(function (col) {
    return col;
  });
}

export default getTableColumns;