import { useRef, useCallback, useEffect } from 'react';
import { getHeight, addStyle, removeStyle, on } from 'dom-lib';
import toggleClass from './toggleClass';
import isNumberOrTrue from './isNumberOrTrue';
import useUpdateEffect from './useUpdateEffect';

var useAffix = function useAffix(props) {
  var tableHeight = props.tableHeight,
      contentHeight = props.contentHeight,
      affixHorizontalScrollbar = props.affixHorizontalScrollbar,
      affixHeader = props.affixHeader,
      tableOffset = props.tableOffset,
      headerOffset = props.headerOffset,
      headerHeight = props.headerHeight,
      scrollbarXRef = props.scrollbarXRef,
      affixHeaderWrapperRef = props.affixHeaderWrapperRef;
  var scrollListener = useRef();
  var handleAffixHorizontalScrollbar = useCallback(function () {
    var _scrollbarXRef$curren;

    var scrollY = window.scrollY || window.pageYOffset;
    var windowHeight = getHeight(window);
    var height = tableHeight();
    var bottom = typeof affixHorizontalScrollbar === 'number' ? affixHorizontalScrollbar : 0;
    var fixedScrollbar = scrollY + windowHeight < height + (tableOffset.current.top + bottom) && scrollY + windowHeight - headerHeight > tableOffset.current.top + bottom;

    if (scrollbarXRef !== null && scrollbarXRef !== void 0 && (_scrollbarXRef$curren = scrollbarXRef.current) !== null && _scrollbarXRef$curren !== void 0 && _scrollbarXRef$curren.root) {
      toggleClass(scrollbarXRef.current.root, 'fixed', fixedScrollbar);

      if (fixedScrollbar) {
        addStyle(scrollbarXRef.current.root, 'bottom', bottom + "px");
      } else {
        removeStyle(scrollbarXRef.current.root, 'bottom');
      }
    }
  }, [affixHorizontalScrollbar, headerHeight, scrollbarXRef, tableHeight, tableOffset]);
  var handleAffixTableHeader = useCallback(function () {
    var top = typeof affixHeader === 'number' ? affixHeader : 0;
    var scrollY = window.scrollY || window.pageYOffset;
    var fixedHeader = scrollY - (headerOffset.current.top - top) >= 0 && scrollY < headerOffset.current.top - top + contentHeight.current;

    if (affixHeaderWrapperRef.current) {
      toggleClass(affixHeaderWrapperRef.current, 'fixed', fixedHeader);
    }
  }, [affixHeader, affixHeaderWrapperRef, contentHeight, headerOffset]);
  var handleWindowScroll = useCallback(function () {
    if (isNumberOrTrue(affixHeader)) {
      handleAffixTableHeader();
    }

    if (isNumberOrTrue(affixHorizontalScrollbar)) {
      handleAffixHorizontalScrollbar();
    }
  }, [affixHeader, affixHorizontalScrollbar, handleAffixTableHeader, handleAffixHorizontalScrollbar]);
  /**
   * Update the position of the fixed element after the height of the table changes.
   * fix: https://github.com/rsuite/rsuite/issues/1716
   */

  useUpdateEffect(handleWindowScroll, [tableHeight]);
  useEffect(function () {
    if (isNumberOrTrue(affixHeader) || isNumberOrTrue(affixHorizontalScrollbar)) {
      scrollListener.current = on(window, 'scroll', handleWindowScroll);
    }

    return function () {
      var _scrollListener$curre;

      (_scrollListener$curre = scrollListener.current) === null || _scrollListener$curre === void 0 ? void 0 : _scrollListener$curre.off();
    }; // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
};

export default useAffix;